@@ -35,8 +35,8 @@ lslxFitting$set("private",
                          data,
                          control) {
                   self$control <- control
-                  if (!(self$control$penalty_method %in% c("none", "lasso", "mcp"))) {
-                    stop("Argument 'penalty_method' can be only either 'none', 'lasso', or 'mcp'.")
+                  if (!(self$control$penalty_method %in% c("none", "lasso", "ridge", "elastic_net", "mcp", "forward", "backward"))) {
+                    stop("Argument 'penalty_method' can be only either 'none', 'lasso', 'ridge', 'elastic_net', 'mcp', 'forward', or 'backward'.")
                   }
                   if (!is.numeric(self$control$lambda_grid)) {
                     if (!is.character(self$control$lambda_grid)) {
@@ -64,6 +64,20 @@ lslxFitting$set("private",
                       }
                     }
                   }
+                  if (!is.numeric(self$control$step_grid)) {
+                    if (!is.character(self$control$step_grid)) {
+                      stop("Argument 'step_grid' can be only a numeric vector or set as 'default'.")
+                    } else if (is.character(self$control$step_grid) &
+                               (length(self$control$step_grid) != 1)) {
+                      stop("Argument 'step_grid' can be only a numeric vector or set as 'default'.")
+                    } else if (is.character(self$control$step_grid) &
+                               (length(self$control$step_grid) == 1)) {
+                      if (self$control$step_grid != "default") {
+                        stop("Argument 'step_grid' can be only a numeric vector or set as 'default'.")
+                      }
+                    }
+                  }
+                  
                   if (!(self$control$loss %in% c("default", "ml", "uls", "dwls", "wls"))) {
                     stop("Argument 'default' can be only 'default', 'ml', 'uls', 'dwls', or 'wls'.")
                   }
@@ -72,7 +86,7 @@ lslxFitting$set("private",
                   }
                   if (!(self$control$missing_method %in% c("default", "two_stage", "listwise_deletion"))) {
                     stop(
-                      "Argument 'start_method' can be only 'default', 'two_stage', or 'listwise_deletion'."
+                      "Argument 'missing_method' can be only 'default', 'two_stage', or 'listwise_deletion'."
                     )
                   }
                   if (!(self$control$start_method %in% c("default", "none", "mh", "heuristic"))) {
@@ -81,7 +95,6 @@ lslxFitting$set("private",
                   if (!(self$control$lambda_direction %in% c("default", "manual", "decrease", "increase"))) {
                     stop("Argument 'lambda_direction' can be only 'default', 'manual', 'decrease', or 'increase'.")
                   }
-                  
                   if (!is.null(self$control$subset)) {
                     if (!(is.integer(self$control$subset) | is.logical(self$control$subset))) {
                       stop("Argument 'subset' must be a integer or logical vector.")
@@ -135,6 +148,9 @@ lslxFitting$set("private",
                   if (!(is.numeric(self$control$minimum_variance) & (length(self$control$minimum_variance) == 1))) {
                     stop("Argument 'minimum_variance' must be a numeric vector with length one.")
                   }
+                  if (!(is.logical(self$control$enforce_cd) & (length(self$control$minimum_variance) == 1))) {
+                    stop("Argument 'enforce_cd' must be a logical vector with length one.")
+                  }
                   if (!is.null(self$control$weight_matrix)) {
                     if (!is.list(self$control$weight_matrix) & !is.matrix(self$control$weight_matrix)) {
                       stop(
@@ -166,7 +182,6 @@ lslxFitting$set("private",
                            "\n  The correct dimension is ", n_moment, " by ", n_moment, ".")
                     }
                   }
-
                   if (length(model$name_factor) == 0) {
                     if (!(c("y<-y") %in% model$specification$block)) {
                       self$control$model_class <- "ca" 
@@ -202,42 +217,80 @@ lslxFitting$set("private",
                   } else {
                     self$control$auxiliary <- FALSE
                   }
-                  if (self$control$penalty_method %in% c("mcp", "lasso")) {
+                  if (self$control$penalty_method %in% c("lasso", "ridge", "elastic_net", "mcp")) {
                     self$control$regularizer <- TRUE
                   } else {
                     self$control$regularizer <- FALSE
                   }
+                  if (self$control$penalty_method %in% c("none", "forward", "backward")) {
+                    self$control$searcher <- TRUE
+                  } else {
+                    self$control$searcher <- FALSE
+                  }
                   if (self$control$penalty_method == "none" & 
                       any(model$specification$type == "pen")) {
                     stop(
                       "When the specified model includes penalized coefficients, 'penalty_method' cannot be 'none'."
                     )
                   }
-                  if (self$control$penalty_method == "none") {
-                    self$control$lambda_grid <- Inf
-                  } else if (self$control$penalty_method %in% c("lasso", "mcp")) {
-                    if (self$control$lambda_grid[[1]] == "default") {
-                    } else {
+                  if (self$control$regularizer) {
+                    self$control$regularizer_type <- self$control$penalty_method
+                    if (self$control$lambda_grid[[1]] != "default") {
                       if (any(self$control$lambda_grid < 0)) {
                         stop(
-                          "When argument 'penalty_method' is set as 'lasso' or 'mcp', any element in argument 'lambda_grid' cannot be smaller than 0."
+                          "When 'penalty_method' is set as 'lasso' or 'mcp', any element in 'lambda_grid' must be non-negative."
                         )
                       }
                     }
-                  } else {}
-                  if (self$control$penalty_method %in% c("none", "lasso")) {
-                    self$control$delta_grid <- Inf
-                  } else if (self$control$penalty_method == "mcp") {
-                    if (self$control$delta_grid[[1]] == "default") {
-                    } else {
-                      if (any(self$control$delta_grid <= 0)) {
+                    if (self$control$delta_grid[[1]] != "default") {
+                      if (self$control$penalty_method %in% c("lasso", "ridge")) {
                         stop(
-                          "When argument 'penalty_method' is set as 'mcp', any element in argument 'delta_grid' must be positive."
+                          "When 'penalty_method' is set as 'lasso' or 'ridge', 'delta_grid' must be set as 'default'."
                         )
-                      }
+                      } else if (self$control$penalty_method == "elastic_net") {
+                        if (any(self$control$delta_grid < 0) | any(self$control$delta_grid > 1)) {
+                          stop(
+                            "When 'penalty_method' is set as 'elastic_net', any element in 'delta_grid' must be in [0, 1]."
+                          )
+                        }
+                      } else if (self$control$penalty_method == "mcp") {
+                        if (any(self$control$delta_grid <= 0)) {
+                          stop(
+                            "When 'penalty_method' is set as 'mcp', any element in 'delta_grid' must be positive."
+                          )
+                        }
+                      } else {}
                     }
                   } else {
+                    self$control$regularizer_type <- "none"
+                    if (self$control$lambda_grid[[1]] != "default") {
+                      stop(
+                        "When 'penalty_method' is set as 'none', 'forward', or 'backward', 'lambda_grid' must be set as 'default'."
+                      )
+                    }
+                    if (self$control$delta_grid[[1]] != "default") {
+                      stop(
+                        "When 'penalty_method' is set as 'none', 'forward', or 'backward', 'delta_grid' must be set as 'default'."
+                      )
+                    }
+                    if (self$control$lambda_direction != "default") {
+                      stop(
+                        "When 'penalty_method' is set as 'none', 'forward', or 'backward', 'lambda_direction' must be set as 'default'."
+                      )
+                    }
                   }
+
+                  if (self$control$searcher) {
+                    self$control$searcher_type <- self$control$penalty_method
+                  } else {
+                    self$control$searcher_type <- "none"
+                    if (self$control$step_grid[[1]] != "default") {
+                      stop(
+                        "When 'penalty_method' is set as 'lasso', 'ridge', 'elastic_net', or mcp', 'step_grid' must be set as 'default'."
+                      )
+                    }
+                  }
+                  
                   if (self$control$loss == "default") {
                     self$control$loss <- "ml"
                   }
@@ -259,6 +312,11 @@ lslxFitting$set("private",
                   if (self$control$start_method == "default") {
                     self$control$start_method <- "mh"
                   }
+                  if (self$control$regularizer & (!self$control$enforce_cd)) {
+                    stop(
+                      "If any regularizer is specified, 'enforce_cd' must be set as TRUE."
+                    )
+                  }
                   if (!is.null(self$control$subset)) {
                     if (self$control$response) {
                       if (is.logical(self$control$subset)) {
@@ -266,7 +324,7 @@ lslxFitting$set("private",
                       } 
                     } else {
                       stop(
-                        "When only moment data is available, argument 'subset' cannot be specified."
+                        "When only moment data is available, 'subset' must be set as NULL."
                       )
                     }
                   } else {
@@ -308,6 +366,8 @@ lslxFitting$set("private",
                         (length(model$name_response) + 3) / 2,
                       n_group = length(model$level_group),
                       n_theta = nrow(model$specification),
+                      n_theta_is_free = NA_real_,
+                      n_theta_is_pen = NA_real_,
                       eta_is_exogenous = model$name_eta %in% model$name_exogenous,
                       eta_is_endogenous = model$name_eta %in% model$name_endogenous,
                       theta_name = rownames(model$specification),
@@ -383,6 +443,10 @@ lslxFitting$set("private",
                         )
                       )
                     )
+                  self$reduced_model$n_theta_is_free <- 
+                    sum(self$reduced_model$theta_is_free)
+                  self$reduced_model$n_theta_is_pen <- 
+                    sum(self$reduced_model$theta_is_pen)
                 })
 
 ## \code{$initialize_reduced_data()} initializes a reduced data. ##
@@ -1309,66 +1373,92 @@ lslxFitting$set("private",
                   } else {
                     eta_is_endogenous <- rep(TRUE, self$reduced_model$n_eta)
                   }
-                  if (self$control$lambda_grid[[1]] == "default") {
-                    if (self$control$start_method == "mh") {
-                      beta_start_inv <- 
-                        solve(diag(self$reduced_model$n_eta) - self$supplied_result$beta_start)
-                      sigma_eta_start <- 
-                        beta_start_inv %*% self$supplied_result$phi_start %*% t(beta_start_inv)
-                      lambda_max <-  
-                        sqrt(quantile(diag(sigma_eta_start)[eta_is_exogenous], 0.6)) / 
-                        quantile(diag(self$supplied_result$phi_start)[eta_is_endogenous] /
-                                   sqrt(diag(sigma_eta_start)[eta_is_endogenous]), 0.4) *
-                        self$control$threshold_value
-                    } else if (self$control$start_method == "heuristic") {
-                      saturated_var <- diag(do.call("+", self$reduced_data$saturated_cov))
-                      lambda_max <- 
-                        1 / quantile((saturated_var * 0.3) / sqrt(saturated_var), 0.3) * 
-                        self$control$threshold_value
-                    } else {
+                  if (self$control$regularizer) {
+                    if (self$control$lambda_grid[[1]] == "default") {
+                      if (self$control$start_method == "mh") {
+                        beta_start_inv <- 
+                          solve(diag(self$reduced_model$n_eta) - self$supplied_result$beta_start)
+                        sigma_eta_start <- 
+                          beta_start_inv %*% self$supplied_result$phi_start %*% t(beta_start_inv)
+                        lambda_max <-  
+                          sqrt(quantile(diag(sigma_eta_start)[eta_is_exogenous], 0.6)) / 
+                          quantile(diag(self$supplied_result$phi_start)[eta_is_endogenous] /
+                                     sqrt(diag(sigma_eta_start)[eta_is_endogenous]), 0.4) *
+                          self$control$threshold_value
+                      } else if (self$control$start_method == "heuristic") {
+                        saturated_var <- diag(do.call("+", self$reduced_data$saturated_cov))
+                        lambda_max <- 
+                          1 / quantile((saturated_var * 0.3) / sqrt(saturated_var), 0.3) * 
+                          self$control$threshold_value
+                      } else {}
+                      lambda_min <- (log(self$reduced_data$n_observation)/ self$reduced_data$n_observation)
+                      self$control$lambda_grid <- 
+                        exp(seq(log(lambda_max), log(lambda_min), 
+                                length.out = self$control$lambda_length))
+                    } 
+                    if (self$control$delta_grid[[1]] == "default") {
+                      if (self$control$penalty_method == "lasso") {
+                        self$control$delta_grid <- 1
+                      } else if (self$control$penalty_method == "ridge") {
+                        self$control$delta_grid <- 0
+                      } else if (self$control$penalty_method == "elastic_net") {
+                        if (self$control$delta_length == 1) {
+                          self$control$delta_grid <- 0.5
+                        } else {
+                          self$control$delta_grid <- seq(0, 1, length.out = self$control$delta_length)
+                        }
+                      } else if (self$control$penalty_method %in% c("mcp")) {
+                        if (self$control$start_method == "mh") {
+                          beta_start_inv <- 
+                            solve(diag(self$reduced_model$n_eta) - self$supplied_result$beta_start)
+                          sigma_eta_start <- 
+                            beta_start_inv %*% self$supplied_result$phi_start %*% t(beta_start_inv)
+                          delta_min <- 
+                            2 * max(diag(sigma_eta_start)[eta_is_endogenous]) / 
+                            min(diag(sigma_eta_start)[eta_is_exogenous])
+                        } else if (self$control$start_method == "heuristic") {
+                          saturated_var <- diag(do.call("+", self$reduced_data$saturated_cov))
+                          delta_min <- 2 * max(saturated_var, 1) / 1
+                        } else {
+                        }
+                        if (self$control$delta_length == 1) {
+                          self$control$delta_grid <- Inf
+                        } else {
+                          self$control$delta_grid <- 
+                            c(delta_min * (1:(self$control$delta_length - 1)), Inf) 
+                        }
+                      } else {}
                     }
-                    lambda_min <- (log(self$reduced_data$n_observation)/ self$reduced_data$n_observation)
-                    self$control$lambda_grid <- 
-                      exp(seq(log(lambda_max), log(lambda_min), 
-                              length.out = self$control$lambda_length))
-                  } 
-                  if (self$control$delta_grid[[1]] == "default") {
-                    if (self$control$start_method == "mh") {
-                      beta_start_inv <- 
-                        solve(diag(self$reduced_model$n_eta) - self$supplied_result$beta_start)
-                      sigma_eta_start <- 
-                        beta_start_inv %*% self$supplied_result$phi_start %*% t(beta_start_inv)
-                      delta_min <- 
-                        2 * max(diag(sigma_eta_start)[eta_is_endogenous]) / 
-                        min(diag(sigma_eta_start)[eta_is_exogenous])
-                    } else if (self$control$start_method == "heuristic") {
-                      saturated_var <- diag(do.call("+", self$reduced_data$saturated_cov))
-                      delta_min <- 2 * max(saturated_var, 1) / 1
-                    } else {
-                    }
-                    if (self$control$delta_length == 1) {
-                      self$control$delta_grid <- Inf
-                    } else {
-                      self$control$delta_grid <- 
-                        c(delta_min * (1:(self$control$delta_length - 1)), Inf) 
+                    if (self$control$lambda_direction == "default") {
+                      if (min(self$control$lambda_grid) == 0) {
+                        self$control$lambda_direction <- "decrease"
+                      } else {
+                        self$control$lambda_direction <- "increase"
+                      }
                     }
+                    if (self$control$lambda_direction == "decrease") {
+                      self$control$lambda_grid <-
+                        sort(self$control$lambda_grid, decreasing = TRUE)
+                    } else if (self$control$lambda_direction == "increase") {
+                      self$control$lambda_grid <-
+                        sort(self$control$lambda_grid, decreasing = FALSE)
+                    } else {}
+                    self$control$delta_grid <-
+                      sort(self$control$delta_grid, decreasing = TRUE)
+                  } else {
+                    self$control$lambda_grid <- 0
+                    self$control$delta_grid <- Inf
                   }
-                  if (self$control$lambda_direction == "default") {
-                    if (min(self$control$lambda_grid) == 0) {
-                      self$control$lambda_direction <- "decrease"
+                  if (self$control$searcher) {
+                    if (self$control$step_grid[[1]] == "default") {
+                      self$control$step_grid <- 
+                        0:self$reduced_model$n_theta_is_pen
                     } else {
-                      self$control$lambda_direction <- "increase"
+                      self$control$step_grid <- 
+                        0:min(self$reduced_model$n_theta_is_pen, 
+                              max(round(self$control$step_grid)))
                     }
-                  }
-                  if (self$control$lambda_direction == "decrease") {
-                    self$control$lambda_grid <-
-                      sort(self$control$lambda_grid, decreasing = TRUE)
-                  } else if (self$control$lambda_direction == "increase") {
-                    self$control$lambda_grid <-
-                      sort(self$control$lambda_grid, decreasing = FALSE)
-                  } else {}
-                  self$control$delta_grid <-
-                    sort(self$control$delta_grid, decreasing = TRUE)
+                  } else {self$control$step_grid = 0}
                 })
 
 
@@ -1377,34 +1467,37 @@ lslxFitting$set("private",
                 "initialize_fitted_result",
                 function() {
                   self$fitted_result <- list()
+                  if (self$control$regularizer) {
+                    length_fitted_result <-  
+                      length(self$control$lambda_grid) * 
+                      length(self$control$delta_grid)
+                  } else {
+                    length_fitted_result <-
+                      length(self$control$step_grid)
+                  }
                   self$fitted_result$numerical_condition <-
                     vector(
                       mode = "list",
-                      length = length(self$control$lambda_grid) *
-                        length(self$control$delta_grid)
+                      length = length_fitted_result
                     )
                   self$fitted_result$information_criterion <-
                     vector(
                       mode = "list",
-                      length = length(self$control$lambda_grid) *
-                        length(self$control$delta_grid)
+                      length = length_fitted_result
                     )
                   self$fitted_result$fit_index <-
                     vector(
                       mode = "list",
-                      length = length(self$control$lambda_grid) *
-                        length(self$control$delta_grid)
+                      length = length_fitted_result
                     )
                   self$fitted_result$cv_error <-
                     vector(
                       mode = "list",
-                      length = length(self$control$lambda_grid) *
-                        length(self$control$delta_grid)
+                      length = length_fitted_result
                     )
                   self$fitted_result$coefficient <-
                     vector(
                       mode = "list",
-                      length = length(self$control$lambda_grid) *
-                        length(self$control$delta_grid)
+                      length = length_fitted_result
                     )
                 })
