@@ -3,32 +3,26 @@ options(digits = 3)
 options(width = 100)
 
 ## ----comment = "", message = FALSE----------------------------------------------------------------
-model <-
-'
-visual  :=> fix(1) * x1 + x2 + x3 
-textual :=> fix(1) * x4 + x5 + x6 
-speed   :=> fix(1) * x7 + x8 + x9 
-'
+model_mgfa <- "visual  :=> 1 * x1 + x2 + x3 
+               textual :=> 1 * x4 + x5 + x6 
+               speed   :=> 1 * x7 + x8 + x9"
 
 ## ----comment = "", message = FALSE----------------------------------------------------------------
 library(lslx)
-r6_lslx <- lslx$new(model = model,
-                     data = lavaan::HolzingerSwineford1939,
-                     group_variable = "school",
-                     reference_group = "Pasteur")
+lslx_mgfa <- lslx$new(model = model_mgfa,
+                    data = lavaan::HolzingerSwineford1939,
+                    group_variable = "school",
+                    reference_group = "Pasteur")
 
 ## ----comment = "", message = FALSE----------------------------------------------------------------
-r6_lslx$penalize_heterogeneity(block = "y<-f", group = "Grant-White")
-r6_lslx$penalize_heterogeneity(block = "y<-1", group = "Grant-White")
+lslx_mgfa$penalize_heterogeneity(block = c("y<-1", "y<-f"), group = "Grant-White")
 
 ## ----comment = "", message = FALSE----------------------------------------------------------------
-r6_lslx$free_directed(left = c("visual", "textual", "speed"),
-                      right = "1",
-                      group = "Grant-White")
+lslx_mgfa$free_block(block = "f<-1", group = "Grant-White")
 
 ## ----comment = "", message = FALSE----------------------------------------------------------------
-r6_lslx$fit_lasso(lambda_grid = seq(.01, .30, .01))
+lslx_mgfa$fit_mcp()
 
 ## ----comment = "", message = FALSE, fig.width = 24, fig.height = 14-------------------------------
-r6_lslx$summarize(selector = "bic")
+lslx_mgfa$summarize(selector = "hbic")
 
